@@ -23,7 +23,7 @@ static char THIS_FILE[] = __FILE__;
 #endif
 
 CAlphabetManager::CAlphabetManager( CDasherModel *pModel, CLanguageModel *pLanguageModel ) 
-  : m_pLanguageModel(pLanguageModel), m_pModel(pModel) {
+  : CNodeManager(0), m_pLanguageModel(pLanguageModel), m_pModel(pModel) {
 }
 
 CDasherNode *CAlphabetManager::GetRoot(CDasherNode *pParent, int iLower, int iUpper, void *pUserData) {
@@ -47,16 +47,16 @@ CDasherNode *CAlphabetManager::GetRoot(CDasherNode *pParent, int iLower, int iUp
 
 
   if(iSymbol == m_pModel->GetSpaceSymbol())
-    pNewNode = new CDasherNode(*m_pModel, pParent, iSymbol, 0, Opts::Special1, iLower, iUpper, m_pLanguageModel, false, iColour);
+    pNewNode = new CDasherNode(pParent, iSymbol, 0, Opts::Special1, iLower, iUpper, m_pLanguageModel, iColour);
   else
-    pNewNode = new CDasherNode(*m_pModel, pParent, iSymbol, 0, Opts::Nodes1, iLower, iUpper, m_pLanguageModel, false, iColour);
+    pNewNode = new CDasherNode(pParent, iSymbol, 0, Opts::Nodes1, iLower, iUpper, m_pLanguageModel, iColour);
   
   pNewNode->SetContext(m_pLanguageModel->CreateEmptyContext()); // FIXME - handle context properly
   pNewNode->m_pNodeManager = this;
   pNewNode->m_bShove = true;
   pNewNode->m_pBaseGroup = m_pModel->GetAlphabet().m_pBaseGroup;
   pNewNode->m_strDisplayText = m_pModel->GetAlphabet().GetDisplayText(iSymbol);
-  //  pNewNode->Seen(true);
+  pNewNode->Seen(true);
 
   return pNewNode;
 }
@@ -106,17 +106,45 @@ void CAlphabetManager::PopulateChildrenWithSymbol( CDasherNode *pNode, int iExis
         ChildScheme = NormalScheme;
       CDasherNode *pNewNode;
 
-      if( newchars[j] == m_pModel->GetControlSymbol() )
-	pNewNode = m_pModel->GetRoot(1, pNode, iLbnd, cum[j], NULL);
+      if(newchars[j] == m_pModel->GetControlSymbol())
+      	pNewNode = m_pModel->GetRoot(1, pNode, iLbnd, cum[j], NULL);
+      else if(newchars[j] == m_pModel->GetStartConversionSymbol()) {
+#ifdef JAPANESE
+      	pNewNode = m_pModel->GetRoot(2, pNode, iLbnd, cum[j], NULL);
+#else
+	pNewNode = m_pModel->GetRoot(0, pNode, iLbnd, cum[j], NULL);
+	pNewNode->Seen(false);
+#endif
+      }
       else if( newchars[j] == iExistingSymbol) {
-	pNewNode = pExistingChild;
-	pNewNode->SetRange(iLbnd, cum[j]);
+      	pNewNode = pExistingChild;
+	      pNewNode->SetRange(iLbnd, cum[j]);
       }
       else {
-	pNewNode = new CDasherNode(*m_pModel, pNode, newchars[j], j, ChildScheme, iLbnd, cum[j], m_pLanguageModel, false, m_pModel->GetColour(newchars[j]));
-	pNewNode->m_pNodeManager = this;
-	pNewNode->m_bShove = true;
-	pNewNode->m_pBaseGroup = m_pModel->GetAlphabet().m_pBaseGroup;
+        int iColour(m_pModel->GetColour(newchars[j]));
+        // This is provided for backwards compatibility. 
+        // Colours should always be provided by the alphabet file
+        if(iColour == -1) {
+          if(newchars[j] == m_pModel->GetSpaceSymbol()) {
+            iColour = 9;
+          }
+          else if(newchars[j] == m_pModel->GetControlSymbol()) {
+            iColour = 8;
+          }
+          else {
+            iColour = (newchars[j] % 3) + 10;
+          }
+        }
+
+        // Loop colours if necessary for the colour scheme
+        if((ChildScheme % 2) == 1 && iColour < 130) {    // We don't loop on high
+          iColour += 130;
+        }
+
+	      pNewNode = new CDasherNode(pNode, newchars[j], j, ChildScheme, iLbnd, cum[j], m_pLanguageModel, iColour);
+	      pNewNode->m_pNodeManager = this;
+	      pNewNode->m_bShove = true;
+	      pNewNode->m_pBaseGroup = m_pModel->GetAlphabet().m_pBaseGroup;
       }
 
       pNewNode->m_strDisplayText = m_pModel->GetAlphabet().GetDisplayText(newchars[j]);
@@ -175,7 +203,7 @@ CDasherNode *CAlphabetManager::RebuildParent(CDasherNode *pNode, int iGeneration
   CDasherNode *pNewNode;
 
   if(vSymbols.size() <= iGeneration + 1) {
-    pNewNode = new CDasherNode(*m_pModel, 0, 0, 0,  Opts::Nodes1, 0, 0, m_pLanguageModel, false, 7);
+    pNewNode = new CDasherNode(0, 0, 0,  Opts::Nodes1, 0, 0, m_pLanguageModel, 7);
     
     CLanguageModel::Context oContext(m_pLanguageModel->CreateEmptyContext());
     m_pModel->EnterText(oContext, ". ");
@@ -202,7 +230,7 @@ CDasherNode *CAlphabetManager::RebuildParent(CDasherNode *pNode, int iGeneration
 
 
 
-  pNewNode = new CDasherNode(*m_pModel, 0, vSymbols[vSymbols.size() - iGeneration - 2], 0, ChildScheme, 0, 0, m_pLanguageModel, false, m_pModel->GetColour(vSymbols[vSymbols.size() - iGeneration - 2]));
+  pNewNode = new CDasherNode(0, vSymbols[vSymbols.size() - iGeneration - 2], 0, ChildScheme, 0, 0, m_pLanguageModel, m_pModel->GetColour(vSymbols[vSymbols.size() - iGeneration - 2]));
   
   CLanguageModel::Context oContext(m_pLanguageModel->CreateEmptyContext());
 
