@@ -3,6 +3,10 @@
 
 #include <string>
 
+#ifndef _WIN32
+#include "../../config.h"
+#endif
+
 // All parameters go into the enums here
 // They are unique across the different types
 enum { 
@@ -31,7 +35,8 @@ enum {
   LP_LM_WORD_ALPHA, LP_USER_LOG_LEVEL_MASK, LP_SPEED_DIVISOR, 
   LP_ZOOMSTEPS, LP_B, LP_S, LP_Z, LP_R, LP_RIGHTZOOM,
   LP_BOOSTFACTOR, LP_AUTOSPEED_SENSITIVITY, LP_SOCKET_PORT, LP_SOCKET_INPUT_X_MIN, LP_SOCKET_INPUT_X_MAX,
-  LP_SOCKET_INPUT_Y_MIN, LP_SOCKET_INPUT_Y_MAX, LP_OX, LP_OY, LP_MAX_Y, LP_INPUT_FILTER, END_OF_LPS
+  LP_SOCKET_INPUT_Y_MIN, LP_SOCKET_INPUT_Y_MAX, LP_OX, LP_OY, LP_MAX_Y, LP_INPUT_FILTER, 
+  LP_CIRCLE_PERCENT, END_OF_LPS
 };
 
 enum {
@@ -169,7 +174,8 @@ static lp_table longparamtable[] = {
   {LP_OX, "OX", PERS, 2048, "X coordinate of crosshair"},
   {LP_OY, "OY", PERS, 2048, "Y coordinate of crosshair"},
   {LP_MAX_Y, "MaxY", PERS, 4096, "Maximum Y coordinate"},
-  {LP_INPUT_FILTER, "InputFilterID", PERS, 3, "Module ID of input filter"}
+  {LP_INPUT_FILTER, "InputFilterID", PERS, 3, "Module ID of input filter"},
+  {LP_CIRCLE_PERCENT, "CirclePercent", PERS, 10, "Percentage of nominal vertical range to use for radius of start circle"}
 };
 
 static sp_table stringparamtable[] = {
@@ -187,7 +193,11 @@ static sp_table stringparamtable[] = {
   {SP_TRAIN_FILE, "TrainingFile", !PERS, "", "Training text for alphabet"},
   {SP_SOCKET_INPUT_X_LABEL, "SocketInputXLabel", PERS, "x", "Label preceding X values for network input"},
   {SP_SOCKET_INPUT_Y_LABEL, "SocketInputYLabel", PERS, "y", "Label preceding Y values for network input"},
+#ifdef WITH_MAEMO
+  {SP_INPUT_FILTER, "InputFilter", PERS, "Stylus Control", "Input filter used to provide the current control mode"},
+#else
   {SP_INPUT_FILTER, "InputFilter", PERS, "Normal Control", "Input filter used to provide the current control mode"},
+#endif
   {SP_INPUT_DEVICE, "InputDevice", PERS, "Mouse Input", "Driver for the input device"}
 };
 
