@@ -1,15 +1,18 @@
 
-#include "WinCommon.h"          // must include pch first
+#include "Common\WinCommon.h"          // must include pch first
+
+#include "../DasherCore/WrapperFactory.h"
 
 #include "Dasher.h"
 #include "../DasherCore/Event.h"
-#include "EditWrapper.h"
-#include "WinUTF8.h"
-#include "DasherWindow.h"
+#include "Common\WinUTF8.h"
+//#include "DasherWindow.h"
 #include "Widgets/Canvas.h"
 #include "Widgets/Slidebar.h"
-#include "EditWrapper.h"
+#include "DasherMouseInput.h"
+#include "Sockets/SocketInput.h"
 
+#include <sys/stat.h>
 
 using namespace std;
 using namespace Dasher;
@@ -22,94 +25,33 @@ using namespace WinUTF8;
 CONST UINT WM_DASHER_EVENT = RegisterWindowMessage(_WM_DASHER_EVENT);
 CONST UINT WM_DASHER_FOCUS = RegisterWindowMessage(_WM_DASHER_FOCUS);
 
-//void AddFiles(LPCWSTR Alphabets, LPCWSTR Colours, CDasherInterface *Interface)
-void CDasher::AddFiles(Tstring Alphabets, Tstring Colours, CDasherInterface *Interface) 
-{
-  using namespace WinHelper;
-  using namespace WinUTF8;
-
-  std::string filename;
-  WIN32_FIND_DATA find;
-  HANDLE handle;
-
-  handle = FindFirstFile(Alphabets.c_str(), &find);
-  if(handle != INVALID_HANDLE_VALUE) {
-    wstring_to_UTF8string(wstring(find.cFileName), filename);
-    Interface->AddAlphabetFilename(filename);
-    while(FindNextFile(handle, &find) != false) {
-      wstring_to_UTF8string(wstring(find.cFileName), filename);
-      Interface->AddAlphabetFilename(filename);
-    }
-    FindClose(handle);
-  }
-
-  handle = FindFirstFile(Colours.c_str(), &find);
-  if(handle != INVALID_HANDLE_VALUE) {
-    wstring_to_UTF8string(find.cFileName, filename);
-    Interface->AddColourFilename(filename);
-    while(FindNextFile(handle, &find) != false) {
-      wstring_to_UTF8string(find.cFileName, filename);
-      Interface->AddColourFilename(filename);
-    }
-    FindClose(handle);
-
-  }
-}
-
 CDasher::CDasher(HWND Parent):m_hParent(Parent) 
 {
   // This class will be a wrapper for the Dasher 'control' - think ActiveX
 
+  m_pEdit = 0;
 
 
-  using namespace WinHelper;
-  using namespace WinUTF8;
-
-  // Get folder names for system and user data.
-  Tstring UserData, AppData;
-  std::string UserData2, AppData2;
-  Tstring Alphabets, Colours;
-  GetUserDirectory(&UserData);
-  GetAppDirectory(&AppData);
-  UserData += TEXT("dasher.rc\\");
-  AppData += TEXT("system.rc\\");
-  CreateDirectory(UserData.c_str(), NULL);      // Try and create folders. Doesn't seem
-  CreateDirectory(AppData.c_str(), NULL);       // to do any harm if they already exist.
-  wstring_to_UTF8string(UserData, UserData2);   // TODO: I don't know if special characters will work.
-  wstring_to_UTF8string(AppData, AppData2);     // ASCII-only filenames are safest. Being English doesn't help debug this...
-
   // Set up COM for the accessibility stuff
   CoInitialize(NULL);
 
-  // Set up Dasher
-  SetStringParameter(SP_SYSTEM_LOC, AppData2);
-  SetStringParameter(SP_USER_LOC, UserData2);
+  
+  
+ 
+ // SetBoolParameter(BP_COLOUR_MODE, true);
+//  ChangeLanguageModel(0);
 
-  Alphabets = UserData;
-  Alphabets += TEXT("alphabet*.xml");
-  Colours = UserData;
-  Colours += TEXT("colour*.xml");
-  AddFiles(Alphabets, Colours, this);
-  Alphabets = AppData;
-  Alphabets += TEXT("alphabet*.xml");
-  Colours = AppData;
-  Colours += TEXT("colour*.xml");
-  AddFiles(Alphabets, Colours, this);
-
-Realize();
-
-  SetBoolParameter(BP_COLOUR_MODE, true);
-  ChangeLanguageModel(0);
-
-// FIXME - we should create our own edit object (as a wrapper to pass stuff outside), rather than relying on being passed one
-
-  m_pEditWrapper = new CEditWrapper;
-
-  ChangeEdit(m_pEditWrapper);
-
-  m_pCanvas = new CCanvas(this);
+  m_pCanvas = new CCanvas(this, m_pEventHandler, m_pSettingsStore);
   m_pCanvas->Create(m_hParent); // TODO - check return 
 
+  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CSocketInput(m_pEventHandler, m_pSettingsStore)));
+  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherMouseInput(m_pEventHandler, m_pSettingsStore, m_pCanvas->getwindow())));
+
+  CreateInput();
+
+ Realize();
+
+
   m_pSlidebar = new CSlidebar(m_hParent, this, ((double)GetLongParameter(LP_MAX_BITRATE))/100.0, m_pCanvas);
 
 
@@ -145,7 +87,6 @@ CDasher::~CDasher(void) {
 
   delete m_pCanvas;
   delete m_pSlidebar;
-  delete m_pEditWrapper;
 }
 
 // Handle periodically poking the canvas to check for user activity.  
@@ -268,24 +209,7 @@ void CDasher::Log()
 }
 
 void Dasher::CDasher::ExternalEventHandler(CEvent* pEvent) {
-
   SendMessage(m_hParent, WM_DASHER_EVENT, 0, (LPARAM)pEvent);
-
-  // Here we send SendMessage calls to the DasherWindow class
-  if( pEvent->m_iEventType == 1 ) {
-    Dasher::CParameterNotificationEvent * pEvt(static_cast < Dasher::CParameterNotificationEvent * >(pEvent));
-    switch(pEvt->m_iParameter) {
-      case BP_DASHER_PAUSED:
-        SendMessage(m_hParent, WM_COMMAND, ID_EDIT_COPY_ALL, 0);
-        break;
-      case SP_DASHER_FONT:
-        break;
-    }
-  }
-  else if((pEvent->m_iEventType >= 2) && (pEvent->m_iEventType <= 5)) {
-    if(m_DashEditbox != NULL)
-      m_DashEditbox->HandleEvent(pEvent);
-  }
 }
 
 // Get the pointer to our user logging object
@@ -314,37 +238,133 @@ bool Dasher::CDasher::GetWindowSize(int* pTop, int* pLeft, int* pBottom, int* pR
 }
 
 void Dasher::CDasher::SetEdit(CDashEditbox * pEdit) {
-  m_pEditWrapper->SetEventHandler(pEdit);
+  // FIXME - we really need to make sure we have a
+  // more sensible way of passing messages out here.
+
+  m_pEdit = pEdit;
 }
 
 void Dasher::CDasher::WriteTrainFile(const std::string &strNewText) {
+  // TODO: Reimplement
   // FIXME - use parameter
-  const string & TrainFile = GetStringParameter(SP_USER_LOC) + GetTrainFile();
-  if(TrainFile == "")
+//  const string & TrainFile = GetStringParameter(SP_USER_LOC) + GetTrainFile();
+//  if(TrainFile == "")
     return;
 
-  if(strNewText == "")
-    return;
+  //if(strNewText == "")
+  //  return;
 
-  Tstring TTrainFile;
-  UTF8string_to_wstring(TrainFile, TTrainFile);
+  //Tstring TTrainFile;
+  //UTF8string_to_wstring(TrainFile, TTrainFile);
 
-  HANDLE hFile = CreateFile(TTrainFile.c_str(),
-                            GENERIC_WRITE, 0, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
+  //HANDLE hFile = CreateFile(TTrainFile.c_str(),
+  //                          GENERIC_WRITE, 0, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
 
-  if(hFile == INVALID_HANDLE_VALUE) {
-    OutputDebugString(TEXT("Can not open file\n"));
-  }
-  else {
-    DWORD NumberOfBytesWritten;
-    SetFilePointer(hFile, 0, NULL, FILE_END);
+  //if(hFile == INVALID_HANDLE_VALUE) {
+  //  OutputDebugString(TEXT("Can not open file\n"));
+  //}
+  //else {
+  //  DWORD NumberOfBytesWritten;
+  //  SetFilePointer(hFile, 0, NULL, FILE_END);
 
-  // Surely there are better ways to write to files than this??
+  //// Surely there are better ways to write to files than this??
 
-    for(unsigned int i = 0; i < strNewText.size(); i++) {
-      WriteFile(hFile, &strNewText[i], 1, &NumberOfBytesWritten, NULL);
+  //  for(unsigned int i = 0; i < strNewText.size(); i++) {
+  //    WriteFile(hFile, &strNewText[i], 1, &NumberOfBytesWritten, NULL);
+  //  }
+
+  //     CloseHandle(hFile);
+  //}
+}
+
+void CDasher::ScanDirectory(const Tstring &strMask, std::vector<std::string> &vFileList) {
+  using namespace WinUTF8;
+
+  std::string filename;
+  WIN32_FIND_DATA find;
+  HANDLE handle;
+
+  handle = FindFirstFile(strMask.c_str(), &find);
+  if(handle != INVALID_HANDLE_VALUE) {
+    wstring_to_UTF8string(wstring(find.cFileName), filename);
+    vFileList.push_back(filename);
+    while(FindNextFile(handle, &find) != false) {
+      wstring_to_UTF8string(wstring(find.cFileName), filename);
+      vFileList.push_back(filename);
     }
-
-       CloseHandle(hFile);
+    FindClose(handle);
   }
-}
\ No newline at end of file
+}
+
+void CDasher::ScanColourFiles(std::vector<std::string> &vFileList) {
+  Tstring Colours;
+
+  // TODO: Is it okay to have duplicate names in the array?
+  std::string strAppData2(GetStringParameter(SP_SYSTEM_LOC));
+  Tstring strAppData;
+
+  WinUTF8::UTF8string_to_wstring(strAppData2, strAppData);
+  
+  Colours = strAppData;
+  Colours += TEXT("colour*.xml");
+  ScanDirectory(Colours, vFileList); 
+
+  std::string strUserData2(GetStringParameter(SP_USER_LOC));
+  Tstring strUserData;
+
+  WinUTF8::UTF8string_to_wstring(strUserData2, strUserData);
+
+  Colours = strUserData;
+  Colours += TEXT("colour*.xml");
+  ScanDirectory(Colours, vFileList); 
+}
+
+void CDasher::ScanAlphabetFiles(std::vector<std::string> &vFileList) {
+  Tstring Alphabets;
+
+  // TODO: Is it okay to have duplicate names in the array?
+  std::string strAppData2(GetStringParameter(SP_SYSTEM_LOC));
+  Tstring strAppData;
+
+  WinUTF8::UTF8string_to_wstring(strAppData2, strAppData);
+  
+  Alphabets = strAppData;
+  Alphabets += TEXT("alphabet*.xml");
+  ScanDirectory(Alphabets, vFileList); 
+
+  std::string strUserData2(GetStringParameter(SP_USER_LOC));
+  Tstring strUserData;
+
+  WinUTF8::UTF8string_to_wstring(strUserData2, strUserData);
+
+  Alphabets = strUserData;
+  Alphabets += TEXT("alphabet*.xml");
+  ScanDirectory(Alphabets, vFileList); 
+}
+
+void CDasher::SetupPaths() {
+  using namespace WinHelper;
+  using namespace WinUTF8;
+
+  Tstring UserData, AppData;
+  std::string UserData2, AppData2;
+  GetUserDirectory(&UserData);
+  GetAppDirectory(&AppData);
+  UserData += TEXT("dasher.rc\\");
+  AppData += TEXT("system.rc\\");
+  CreateDirectory(UserData.c_str(), NULL);      // Try and create folders. Doesn't seem
+  CreateDirectory(AppData.c_str(), NULL);       // to do any harm if they already exist.
+  wstring_to_UTF8string(UserData, UserData2);   // TODO: I don't know if special characters will work.
+  wstring_to_UTF8string(AppData, AppData2);     // ASCII-only filenames are safest. Being English doesn't help debug this...
+  SetStringParameter(SP_SYSTEM_LOC, AppData2);
+  SetStringParameter(SP_USER_LOC, UserData2);
+}
+
+void CDasher::SetupUI() {
+}
+
+int CDasher::GetFileSize(const std::string &strFileName) {
+  struct _stat sStatInfo;
+  _stat(strFileName.c_str(), &sStatInfo);
+  return sStatInfo.st_size;
+}
